@@ -6,7 +6,9 @@ use base qw/Data::Visitor/;
 use strict;
 use warnings;
 
-__PACKAGE__->mk_accessors( qw/callbacks ignore_return_values/ );
+use Scalar::Util qw/blessed/;
+
+__PACKAGE__->mk_accessors( qw/callbacks class_callbacks ignore_return_values/ );
 
 sub new {
 	my ( $class, %callbacks ) = @_;
@@ -16,9 +18,12 @@ sub new {
 		$ignore_ret = delete $callbacks{ignore_return_values};
 	}
 
+	my @class_callbacks = grep { $_->can("isa") } keys %callbacks;
+
 	$class->SUPER::new({
 		ignore_return_values => $ignore_ret,
 		callbacks => \%callbacks,
+		class_callbacks => \@class_callbacks,
 	});
 }
 
@@ -37,7 +42,19 @@ sub visit_value {
 
 sub visit_object {
 	my ( $self, $data ) = @_;
-	$self->callback( object => $data );
+
+	my $ignore = $self->ignore_return_values;
+
+	my $new_data = $self->callback( object => $data );
+	$data = $new_data unless $ignore;
+
+	foreach my $class ( @{ $self->class_callbacks } ) {
+		last unless blessed($data);
+		my $new_data = $self->callback( $class => $data ) if $data->isa($class);
+		$data = $new_data unless $ignore;
+	}
+
+	$data;
 }
 
 BEGIN {
@@ -48,9 +65,9 @@ BEGIN {
 				my ( $self, $data ) = @_;
 				my $new_data = $self->callback( '.$reftype.' => $data );
 				if ( ref $data eq ref $new_data ) {
-					$self->SUPER::visit_'.$reftype.'( $new_data );
+					return $self->SUPER::visit_'.$reftype.'( $new_data );
 				} else {
-					$self->SUPER::visit( $new_data );
+					return $self->SUPER::visit( $new_data );
 				}
 			}
 		' || die $@;
@@ -61,7 +78,7 @@ sub callback {
 	my ( $self, $name, $data ) = @_;
 
 	if ( my $code = $self->callbacks->{$name} ) {
-		my $ret = $code->( $self, $data );
+		my $ret = $self->$code( $data );
 		return $self->ignore_return_values ? $data : $ret ;
 	} else {
 		return $data;
@@ -157,6 +174,11 @@ Called after C<value> for non references.
 
 Called for blessed objects.
 
+=item Some::Class
+
+You can use any class name as a clalback. This is clled only after the
+C<object> callback.
+
 =item array
 
 Called for array references.
